@@ -22,42 +22,30 @@
 #ifndef BOX_DB_TABLE_H
 #define BOX_DB_TABLE_H
 
-#include "box_db.h"
+#include <glom/box_withbuttons.h>
+#include <glom/base_db_table.h>
 #include <glom/libglom/data_structure/field.h>
+#include <libglademm.h>
 #include <algorithm> //find_if used in various places.
 
+namespace Glom
+{
 
-class Box_DB_Table : public Box_DB
+/** A Box that has access to a database table's structure.
+ */
+class Box_DB_Table
+: public Box_WithButtons,
+  public Base_DB_Table
 {
 public: 
   Box_DB_Table();
   Box_DB_Table(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade);
   virtual ~Box_DB_Table();
-
-  virtual bool init_db_details(const Glib::ustring& table_name);
-  virtual bool refresh_data_from_database();
-
-  virtual Glib::ustring get_table_name();
-
-  //TODO: Put this somewhere more sensible:
-  typedef std::map<Gnome::Gda::ValueType, Glib::ustring> type_map_valuetypes;
-
-protected:
-
-  //virtual Glib::RefPtr<Gnome::Gda::DataModel> record_new(Gnome::Gda::Value primary_key_value);
-
-  Gnome::Gda::Value get_entered_field_data_field_only(const sharedptr<const Field>& field) const;
-  virtual Gnome::Gda::Value get_entered_field_data(const sharedptr<const LayoutItem_Field>& field) const;
-
-  //static sharedptr<Field> get_field_primary_key(const type_vecFields& fields);
-
-  unsigned long get_last_auto_increment_value(const Glib::RefPtr<Gnome::Gda::DataModel>& data_model, const Glib::ustring& field_name);
-
-
-  //static type_vecFields get_fields_for_datamodel(const Glib::RefPtr<Gnome::Gda::DataModel>& data_model); 
-  static Glib::ustring postgres_get_field_definition_for_sql(const Gnome::Gda::FieldAttributes& field_info);
-
-  Glib::ustring m_table_name;
+    
+  Gtk::Window* get_app_window();
+  const Gtk::Window* get_app_window() const;
 };
 
+} //namespace Glom
+
 #endif //BOX_DB_TABLE_H
