@@ -28,10 +28,7 @@
 #include <glibmm/i18n.h>
 #include <gtkmm/messagedialog.h>
 
-#ifndef GLOM_ENABLE_MAEMO
-#include <libgnome/gnome-url.h>
-#include <libgnome/gnome-help.h>
-#endif
+#include <gio/gio.h> // For g_app_info_launch_default_for_uri
 
 #ifdef GLOM_ENABLE_MAEMO
 #include <hildonmm/note.h>
@@ -51,12 +48,49 @@
 #include <stack>
 
 #ifdef GLOM_ENABLE_MAEMO
+//We use different spacings on Maemo because the screen is smaller:
 const unsigned int Glom::Utils::DEFAULT_SPACING_LARGE =  1;
 const unsigned int Glom::Utils::DEFAULT_SPACING_SMALL =  1;
 #else
 const unsigned int Glom::Utils::DEFAULT_SPACING_LARGE = 12;
 const unsigned int Glom::Utils::DEFAULT_SPACING_SMALL =  6;
-#endif
+#endif //GLOM_ENABLE_MAEMO
+
+namespace
+{
+
+// Basically copied from libgnome (gnome-help.c, Copyright (C) 2001 Sid Vicious
+// Copyright (C) 2001 Jonathan Blandford <jrb@alum.mit.edu>), but C++ified
+std::string locate_help_file(const std::string& path, const std::string& doc_name)
+{
+  // g_get_language_names seems not to be wrapped by glibmm
+  const char* const* lang_list = g_get_language_names ();
+
+  for(unsigned int j = 0; lang_list[j] != NULL; ++j)
+  {
+    const char* lang = lang_list[j];
+
+    /* This has to be a valid language AND a language with
+     * no encoding postfix.  The language will come up without
+     * encoding next. */
+    if (lang == NULL || strchr(lang, '.') != NULL)
+      continue;
+
+    const char* exts[] = { "", ".xml", ".docbook", ".sgml", ".html", NULL };
+    for(unsigned i = 0; exts[i] != NULL; ++i)
+    {
+      std::string name = doc_name + exts[i];
+      std::string full = Glib::build_filename(path, Glib::build_filename(lang, name));
+
+      if(Glib::file_test(full, Glib::FILE_TEST_EXISTS))
+        return full;
+    }
+  }
+
+  return std::string();
+}
+
+}
 
 namespace Glom
 {
@@ -278,27 +312,20 @@ Glib::ustring Utils::build_sql_select_with_where_clause(const Glib::ustring& tab
     if(is_summary)
       one_sql_part += fieldsummary->get_summary_type_sql() + "(";
 
-    one_sql_part += ( "\"" + layout_item->get_sql_table_or_join_alias_name(table_name) + "\"." );
+    one_sql_part += layout_item->get_sql_name(table_name);
 
-    const Glib::ustring name = layout_item->get_name();
-    if(!name.empty())
+    //Close the summary bracket if necessary.
+    if(is_summary)
+      one_sql_part +=  ")";
+
+    //Append it to the big string of fields:
+    if(!one_sql_part.empty())
     {
-      one_sql_part += ("\"" + name + "\"");
+      if(!sql_part_fields.empty())
+        sql_part_fields += ", ";
 
-      //Close the summary bracket if necessary.
-      if(is_summary)
-        one_sql_part +=  ")";
-
-      //Append it to the big string of fields:
-      if(!one_sql_part.empty())
-      {
-        if(!sql_part_fields.empty())
-          sql_part_fields += ", ";
-
-        sql_part_fields += one_sql_part;
-      }
+      sql_part_fields += one_sql_part;
     }
-
   }
 
   if(sql_part_fields.empty())
@@ -461,7 +488,7 @@ Utils::type_list_values_with_second Utils::get_choice_values(const sharedptr<con
       itempair.first = datamodel->get_value_at(0, row);
 
       if(with_second)
-                        itempair.second = datamodel->get_value_at(1, row);
+        itempair.second = datamodel->get_value_at(1, row);
 
       list_values.push_back(itempair);
     }
@@ -798,6 +825,7 @@ int Utils::dialog_run_with_help(Gtk::Dialog* dialog, const Glib::ustring& id)
     show_help(id);
     result = dialog->run();
   }
+
   dialog->hide();
   return result;
 }
@@ -811,6 +839,8 @@ int Utils::dialog_run_with_help(Gtk::Dialog* dialog, const Glib::ustring& id)
 
 void Utils::show_help(const Glib::ustring& id)
 {
+  // TODO_maemo: Show help on maemo by some other means
+#ifndef GLOM_ENABLE_MAEMO
   GError* err = 0;
   const gchar* pId;
   if (id.length())
@@ -820,18 +850,36 @@ void Utils::show_help(const Glib::ustring& id)
   else
   {
     pId = 0;
-   }
+  }
 
-  // TODO_maemo: Show help on maemo by some other means
-#ifndef GLOM_ENABLE_MAEMO
-  if (!gnome_help_display("glom.xml", pId, &err))
+  try
   {
-     std::string message = std::string(_("Could not display help: ")) + err->message;
-     Gtk::MessageDialog* dialog = new Gtk::MessageDialog(message, false, Gtk::MESSAGE_ERROR);
-     dialog->run();
-     delete dialog;
-     g_error_free(err);
-   }
+    const char* path = DATADIR "/gnome/help/glom";
+    std::string help_file = locate_help_file(path, "glom.xml");
+    if(help_file.empty())
+    {
+      throw std::runtime_error(_("No help file available"));
+    }
+    else
+    {
+      std::string uri = "ghelp:" + help_file;
+      if(pId) { uri += "?"; uri += pId; }
+
+      // g_app_info_launch_default_for_uri seems not to be wrapped by giomm
+      if(!g_app_info_launch_default_for_uri(uri.c_str(), NULL, &err))
+      {
+        std::string message(err->message);
+        g_error_free(err);
+        throw std::runtime_error(message);
+      }
+    }
+  }
+  catch(const std::exception& ex)
+  {
+    const Glib::ustring message = _("Could not display help: ") + Glib::ustring(ex.what());
+    Gtk::MessageDialog dialog(message, false, Gtk::MESSAGE_ERROR);
+    dialog.run();
+  }
 #endif
 }
 
@@ -839,7 +887,7 @@ void Utils::show_help(const Glib::ustring& id)
 void Utils::show_ok_dialog(const Glib::ustring& title, const Glib::ustring& message, Gtk::Window& parent, Gtk::MessageType message_type)
 {
 #ifdef GLOM_ENABLE_MAEMO
-  // TODO_maemo: Map message_type to an appropriate stock_id?
+  // TODO_maemo: Map message_type to a senseful stock_id?
   Hildon::Note dialog(Hildon::NOTE_TYPE_INFORMATION, parent, message);
 #else
   Gtk::MessageDialog dialog("<b>" + title + "</b>", true /* markup */, message_type, Gtk::BUTTONS_OK);
@@ -849,4 +897,37 @@ void Utils::show_ok_dialog(const Glib::ustring& title, const Glib::ustring& mess
   dialog.run();
 }
 
+namespace
+{
+
+static void on_window_hide(Glib::RefPtr<Glib::MainLoop> main_loop, sigc::connection handler_connection)
+{
+  handler_connection.disconnect(); //This should release a main_loop reference.
+  main_loop->quit();
+
+  //main_loop should be destroyed soon, because nothing else is using it.
+}
+
+} //anonymous namespace.
+
+void Utils::show_window_until_hide(Gtk::Window* window)
+{
+  if(!window)
+    return;
+
+  Glib::RefPtr<Glib::MainLoop> main_loop = Glib::MainLoop::create(false /* not running */);
+
+  //Stop the main_loop when the window is hidden:
+  sigc::connection handler_connection; //TODO: There seems to be a crash if this is on the same line.
+  handler_connection = window->signal_hide().connect( 
+    sigc::bind(
+      sigc::ptr_fun(&on_window_hide),
+      main_loop, handler_connection
+    ) );
+  
+  window->show();
+  main_loop->run(); //Run and block until it is stopped by the hide signal handler.
+}
+
+
 } //namespace Glom
