@@ -440,6 +440,9 @@ bool DbTreeModel::refresh_from_database(const FoundSet& found_set)
 #endif // GLIBMM_EXCEPTIONS_ENABLED
   }
 
+  if(m_found_set.m_table_name.empty())
+    std::cerr << "DEBUG: refresh_from_database(): found_set.m_table_name is empty." << std::endl;
+    
   if(m_connection && !m_found_set.m_table_name.empty() && m_get_records)
   {
     const Glib::ustring sql_query = Utils::build_sql_select_with_where_clause(m_found_set.m_table_name, m_column_fields, m_found_set.m_where_clause, m_found_set.m_extra_join, m_found_set.m_sort_clause, m_found_set.m_extra_group_by);
